@@ -49,11 +49,8 @@ def set_owner_description(toplevel, gitdir, owner, description):
         repo.description = description
 
     if owner is not None:
-        rcw = repo.config_writer()
         logger.debug('Setting %s owner to: %s' % (gitdir, owner))
-        if not rcw.has_section('gitweb'):
-            rcw.add_section('gitweb')
-        rcw.set('gitweb', 'owner', owner)
+        repo.git.config('gitweb.owner', owner)
 
 def set_agefile(toplevel, gitdir, last_modified):
     # set agefile, which can be used by cgit to show idle times
@@ -204,12 +201,17 @@ def write_projects_list(manifest, config):
     (dirname, basename) = os.path.split(plpath)
     (fd, tmpfile) = tempfile.mkstemp(prefix=basename, dir=dirname)
     logger.info('Writing new %s' % plpath)
+
     try:
         fh = open(tmpfile, 'w')
         for gitdir in manifest.keys():
             if trimtop and gitdir.find(trimtop) == 0:
                 gitdir = gitdir[len(trimtop):]
+
+            # Always remove leading slash, otherwise cgit breaks
+            gitdir = gitdir.lstrip('/')
             fh.write('%s\n' % gitdir)
+
         fh.close()
         os.chmod(tmpfile, 0644)
         shutil.move(tmpfile, plpath)
@@ -385,9 +387,21 @@ def pull_mirror(name, config, opts):
     hookscript = config['post_update_hook']
 
     for gitdir in to_pull:
-        pull_repo(toplevel, gitdir)
-        set_agefile(toplevel, gitdir, culled[gitdir]['modified'])
-        run_post_update_hook(hookscript, toplevel, gitdir)
+        # Check in case grok-fsck is checking this repo right now
+        try:
+            fullpath = os.path.join(toplevel, gitdir.lstrip('/'))
+            grokmirror.lock_repo(fullpath, nonblocking=True)
+            pull_repo(toplevel, gitdir)
+            set_agefile(toplevel, gitdir, culled[gitdir]['modified'])
+            run_post_update_hook(hookscript, toplevel, gitdir)
+        except IOError, ex:
+            logger.info('Could not obtain exclusive lock on %s' % gitdir)
+            logger.info('\tAssuming grok-fsck is running, will try later.')
+            # To make sure we check this again during next run,
+            # fudge the manifest accordingly.
+            culled[gitdir] = mymanifest[gitdir]
+            # this is rather hackish, but effective
+            last_modified -= 1
 
     if to_clone:
         # we use "existing" to track which repos can be used as references
@@ -430,6 +444,9 @@ def pull_mirror(name, config, opts):
                 target = os.path.join(config['toplevel'], symlink.lstrip('/'))
                 if not os.path.exists(target):
                     logger.info('Symlinking %s -> %s' % (target, source))
+                    # Make sure the leading dirs are in place
+                    if not os.path.exists(os.path.dirname(target)):
+                        os.makedirs(os.path.dirname(target))
                     os.symlink(source, target)
 
     if opts.purge:
